#! /usr/bin/perl
#
# fpgg.pl - Feline Picture Gallery Generator
# Copyright (C) 2006-2007 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Version: fpgg.pl v0.6.0  09-Jun-2007  wnd@iki.fi

use strict;
use warnings FATAL => 'all';
use diagnostics;

use File::stat;


# tagval includes global variables:
#   TOTAL-PICTURES
#
my %tagval;

$tagval{'TITLE'} = "Yet Another Gallery";
my $main = "index.html";

my $thumb_size = "128x96";
my $view_size = "640x480";
my $view_quality = 85;

my $img_basepath = "";

my ($thumb_width, $thumb_height);
my ($view_width, $view_height);
my ($thumb_true_size, $view_true_size);

my $cleanup = 0;
my $trust_size = 1;

my $template_view = join("\n", &read_file("view.template", 1));
my $template_index = join("\n", &read_file("index.template", 1));
my $template_category = join("\n", &read_file("category.template", 1));



my @file_id;
my %file_name;
my %file_dir;
my %file_cat;
my %file_md5;
my %file_thumb_w;
my %file_thumb_h;
my %categories;

# size, md5, width, height, id, name
my %file_cache;
my @get_info_cache = ();
my $get_info_cache_file = "";



=item print_usage
Print usage and quit
=cut
sub print_usage() {
	print <<EOF;
Usage: fpgg.pl [OPTION ...] dir ...

  --title x		Gallery title

  --thumb-size		Thumbnail dimensions (WIDTHxHEIGHT)
  --view-size		Picture dimensions in view-page (WIDTHxHEIGHT)
  --view-quality	Picture quality (0-100, 100 = best)

  --img-basepath URI	Define hardcoded basepath for images
			(Allows to have images on an external server)
EOF
	exit(0);
}



=item parse_options
Parse command line options
=cut
sub parse_options() {
	while (defined($ARGV[0]) && ($ARGV[0] =~ /^--./)) {
		if ($ARGV[0] eq "--title") {
			shift @ARGV;
			if (! defined($ARGV[0])) {
				print "title missing\n";
				exit(1);
			}
			$tagval{'TITLE'} = $ARGV[0];
			shift @ARGV;
		}

		elsif ($ARGV[0] eq "--view-size") {
			shift @ARGV;
			if (! defined($ARGV[0])) {
				print "view-size missing\n";
				exit(1);
			}
			$view_size = $ARGV[0];
			shift @ARGV;
		}

		elsif ($ARGV[0] eq "--thumb-size") {
			shift @ARGV;
			if (! defined($ARGV[0])) {
				print "thumb-size missing\n";
				exit(1);
			}
			$thumb_size = $ARGV[0];
			shift @ARGV;
		}

		elsif ($ARGV[0] eq "--view-quality") {
			shift @ARGV;
			if (! defined($ARGV[0])) {
				print "view-quality missing\n";
				exit(1);
			}
			$view_quality = $ARGV[0];
			shift @ARGV;
		}

		elsif ($ARGV[0] eq "--img-basepath") {
			shift @ARGV;
			if (! defined($ARGV[0])) {
				print "img-basepath missing\n";
				exit(1);
			}
			$img_basepath = $ARGV[0];
			$img_basepath .= "/" unless ($img_basepath =~ /\/$/);
			shift @ARGV;
		}

		else {
			print "Unknown option: $ARGV[0]\n";
			exit(1);
		}
	}
}



=item read_file
Read file

0: filename
1: failure is fatal
r: (lines without linefeed or undef)
=cut
sub read_file($$) {
	my ($file, $fatal) = @_;
	my $r = open(FILE, "< $file");
	if (! defined($r)) {
		die "$!" if ($fatal == 1);
		return undef;
	}
	my @t = <FILE>;
	chomp @t;
	return @t;
}



=item get_tagval_thumbs
Get tagval for ALL-THUMBS
=cut
sub get_tagval_all_thumbs() {
	my $t = "";
	foreach my $num (@file_id) {
		$t .= &create_thumb_link($num) . "\n";
	}
	$tagval{'ALL-THUMBS'} = $t;
}



=item get_tagval_all_categories
Get tagval for ALL-CATEGORIES
=cut
sub get_tagval_all_categories() {
	my @cats = keys %categories;
	if ($#cats <= 0) {
		$tagval{'ALL-CATEGORIES'} = "";
		return;
	}

	my $r = "<ul>\n";
	foreach my $cat (sort keys %categories) {
		my $catfile = &make_cat_file($cat);
		$r .= "\t<li><a href=\"$catfile\">$cat</a></li>\n";
	}
	$r .= "</ul>\n";
	$tagval{'ALL-CATEGORIES'} = $r;
}



=item replace_simple_tags
Replace simple tags in data

0: data
r: data
=cut
sub replace_simple_tags($) {
	$_ = shift;

	foreach my $tag ("TITLE", "TOTAL-PICTURES", "CREATED",
			"ALL-THUMBS", "ALL-CATEGORIES",
			"CATEGORY", "CATEGORY-THUMBS",
			"VIEW-OPTIONS",
			"PICTURE-FULLPATH", "PICTURE-FILE", "PICTURE-PATH",
			"PICTURE-TITLE", "PICTURE-DESCRIPTION",
			"PICTURE-CATEGORIES", "PICTURE-I",
			"PICTURE-VIEW", "PICTURE-VIEW-LINK-ORIG",
			"PICTURE-VIEW-LINK-NEXT",
			"ALL-THUMBS", "ALL-CATEGORIES") {
		my $val = $tagval{$tag};
		if (! defined($val)) {
			s/(.*\n).*?<\*TAG:\Q$tag\E>.*?\n/$1/gs;
		} else {
			s/<\*?TAG:\Q$tag\E>/$val/gs;
		}
	}
	return $_;
}



=item replace_opt_tags
Replace tags with options

0: data
r: data
=cut
sub replace_opt_tags($) {
	$_ = shift;

	# replace single option tags
	foreach my $tag ("VIEW-LINK-PREV", "VIEW-LINK-NEXT",
			"LINK-INDEX", "PICTURE-LINK-ORIGINAL") {
		my $replace;
		# replace tags with option
		foreach my $macro (/(<TAG:$tag:[^>]+>)/gs) {
			my ($label) = $macro =~ /.*:(.*)>/s;
			$replace = sprintf("<a href=\"%s\">%s</a>",
					$tagval{"$tag-URL"}, $label);
			s/\Q$macro\E/$replace/s;
		}

		# replace remaining tags
		my $url = $tagval{"$tag-URL"};
		if (defined($url)) {
			$replace = sprintf("<a href=\"%s\">%s</a>",
					$url, $tagval{"$tag-LABEL"});
		} else {
			$replace = $tagval{"$tag-LABEL"};
		}
		s/<TAG:\Q$tag\E>/$replace/gs;
	}

	# replace multi-option tags
	foreach my $macro (/(<\*?TAG:PICTURE-MAP[^>]*>)/gs) {
		my @t = split(/[<>:]/, $macro);
		my $level = $tagval{"PICTURE-MAP-LEVEL"};
		my $label = ($level == 1) ? "Location" : "Approximate location";
		$label = $t[($level + 2)] if (defined($t[($level + 2)]));
		if ($level == 0 || ! defined($tagval{'PICTURE-MAP-URL'})) {
			if ($t[1] =~ /^\*/) {
				s/(.*\n).*?<\*TAG:PICTURE-MAP[^>]*>.*?\n/$1/gs;
			} else {
				s/\Q$macro\E//s;
			}
		} else {
			my $val = sprintf("<a href=\"%s\">%s</a>\n",
					$tagval{'PICTURE-MAP-URL'},
					$label);
			s/\Q$macro\E/$val/s;
		}
	}
	
	return $_;
}



=item replace_tags
Replace tags

0: data
r: data
=cut
sub replace_tags($) {
	my ($t) = @_;
	$t = &replace_simple_tags($t);
	$t = &replace_opt_tags($t);
	return $t;
}



=item get_size
Get file size

0: path
r: size
=cut
sub get_size($) {
	my ($file) = @_;
	my $sb = stat($file);
	return $sb->size;
}



=item get_md5
Get file MD5

0: file
1: md5sum
=cut
sub get_md5($) {
	my ($file) = @_;
	open(MPIPE, "md5sum $file |") or die $@;
	my $line = <MPIPE>;
	close(MPIPE);
	my ($md5) = $line =~ /(.*?) /;
	return $md5;
}



=cut cache_get_file_id
Get file ID from cache

0: picture file
r: ID
=cut
sub cache_get_file_id($) {
	my ($file) = @_;
	my @r = &cache_get($file);
	return $r[4];
}



=item cache_get
Get picture info from cache

0: picture file
=cut
sub cache_get($) {
	my ($file) = @_;
	my $line = $file_cache{$file};
	return () if (! defined($line));

	my ($size, $md5, $width, $height, $id) = $line =~
			/(\d+)\t([0-9a-f]+)\t(\d+)\t(\d+)\t(\d+)/i;
	return ($size, $md5, $width, $height, $id);
}



=item cache_set
Write file info into cache

0: file
1: file size
2: md5
3: width
4: height
5: number
=cut
sub cache_set($$$$$$) {
	my ($file, $size, $md5, $width, $height, $num) = @_;
	$file_cache{$file} = "$size\t$md5\t$width\t$height\t$num";
}



=item get_file_info
Get image info

0: directory
1: file
2: global id
r: (id, width, height, need_rebuild)
=cut
sub get_file_info($$$) {
	my ($dir, $file, $gid) = @_;
	my ($size, $md5, $width, $height, $id) = &cache_get("$dir/$file");
	my $rebuild;

	if (defined($id)) {
		if ($trust_size == 1) {
			my $tsize = &get_size("$dir/$file");
			$rebuild = ($tsize == $size) ? 0 : 1;
		} else {
			my $tmd5 = &get_md5("$dir/$file");
			$rebuild = ($tmd5 eq $md5) ? 0 : 1;
		}
	}
	if (! defined($id) || $rebuild == 1) {
		$size = &get_size("$dir/$file");
		print ".";
		$md5 = &get_md5("$dir/$file");
		print ".";
		($width, $height) = &get_dimensions("$dir/$file");
		print ".";
		$id = $gid;
		&cache_set("$dir/$file", $size, $md5, $width, $height, $id);
		$rebuild = 1;
	}
	return ($id, $width, $height, $rebuild);
}



=item get_new_id
Get new ID
TODO: what is this?-)

r: ID
=cut
sub get_new_id() {
	my $big_id = -1;
	while (my ($key, $val) = each %file_cache) {
		my $id = &cache_get_file_id($key);
		$big_id = $id if ($id > $big_id);
	}
	return $big_id + 1;
}



=item cache_load
Load picture information cache

0: file
=cut
sub cache_load($) {
	my ($file) = @_;
	open(CFILE, "< $file") or return -1;
	while(<CFILE>) {
		my ($val, $key) = /(.*)\t(.*)/;
		$file_cache{$key} = $val;
	}
	close(CFILE);
}



=item cache_save
Save cached information

0: file
=cut
sub cache_save($) {
	my ($file) = @_;
	open(CFILE, ">$file") or die $@;
	while (my ($key, $val) = each %file_cache) {
		print CFILE "$val\t$key\n";
	}
	close(CFILE);
}



=item split_categories
Split string into categories

0: string
r: (categories)
=cut
sub split_categories($) {
	my ($catstr) = @_;
	return () if (! defined($catstr));
	my @cats = split(/,/, $catstr);
	foreach (@cats) {
		s/^ *//;
		s/ *$//;
	}
	return @cats;
}



=item add_to_cats
Add image to categories

0: filename
1: directory
2: number
3: categories
4: thumb width
5: thumb height
=cut
sub add_to_cats($$$$$$) {
	my ($file, $dir, $num, $cats, $width, $height) = @_;
	$categories{$_} .= "$num " foreach (&split_categories($cats));

	die "sanity check" if (defined($file_name{$num}));
	$file_name{$num} = $file;

	die "sanity check" if (defined($file_dir{$num}));
	$file_dir{$num} = $dir;
	
	die "sanity check" if (defined($file_cat{$num}));
	if (defined($cats)) {
		$file_cat{$num} = "$cats, All";
	} else {
		$file_cat{$num} = "All";
	}
	
	die "sanity check" if (defined($file_thumb_w{$num}));
	$file_thumb_w{$num} = $width;

	die "sanity check" if (defined($file_thumb_h{$num}));
	$file_thumb_h{$num} = $height;

	push(@file_id, $num);
}



=item create_thumb_link
Create thumbnail link

0: image number
r: string
=cut
sub create_thumb_link($) {
	my ($num) = @_;

	my $dir = $file_dir{$num};
	my $file = $file_name{$num};

	my $thumb = &make_thumb_name($num, $dir, $file);
	my $view = &make_view_html_name($num);
	my $twidth = $file_thumb_w{$num};
	my $theight = $file_thumb_h{$num};
	my $alt = &get_info($dir, $file, "title");
	$alt = "$dir/$file" unless defined($alt);

	my $class;
	if ($twidth > $theight) {
		$class = "vpad";
	} else {
		$class = "hpad";
	}
	return "<a href=\"$view\"><img src=\"$img_basepath$thumb\"\n" .
			"\tclass=\"$class\" " .
				"width=\"$twidth\" height=\"$theight\"\n" .
			"\talt=\"$alt\" /></a>";
}



=item build_category
Build category page

0: category
=cut
sub build_category($) {
	my ($cat) = @_;

	$tagval{'CATEGORY'} = $cat;

	my $t = "";
	foreach my $num (split(/ /, $categories{$cat})) {
		$t .= &create_thumb_link($num) . "\n";
	}
	$tagval{'CATEGORY-THUMBS'} = $t;

	my $out_file = &make_cat_file($cat);
	my $page = $template_category;

	$page = &replace_tags($page);

	open(FILE, "> $out_file") or die $@;
	print FILE $page;
	close(FILE);
}



=item build_cats
Build category pages
=cut
sub build_cats() {
	&build_category($_) foreach (keys %categories);
}



=item get_info
Get info of picture

0: directory
1: file
2: field
r: info
=cut
sub get_info($$$) {
	my ($dir, $file, $field) = @_;
	my $phase = 0;
	my $ret;
	if ($get_info_cache_file ne "$dir/info") {
		@get_info_cache = &read_file("$dir/info", 0);
		return undef unless defined($get_info_cache[0]);
		$get_info_cache_file = "$dir/info";
	}

	foreach (@get_info_cache) {
		if (/^file: /) {
			($phase != 0) && return $ret;
			(/^file: $file/) || next;
			$phase = 1;
			next;
		}
		if ($phase == 1 && /^$field:/) {
			($ret) = /.*?: (.*)/;
			$phase = 2;
			next;
		}
		if ($phase == 2 && (/^$/ || /^[^ ]+:/)) {
			close(FILE);
			return $ret;
		}
		$ret .= "\n$_" if ($phase == 2);
	}
	return $ret;
}



=item get_dimensions
Get image dimensions

0: file
r: (width, height)
=cut
sub get_dimensions($$) {
	my ($file) = @_;
	open(SPIPE, "identify \"$file\" |") or die "pipe failed!";
	my $out = <SPIPE>;
	close(SPIPE);
	my ($width, $height) = $out =~ /.* (\d+)x(\d+)/;
	return ($width, $height);
}



=item scale_size
Scale dimensions

0: original width
1: priginal height
2: mode
r: (width, height)
=cut
sub scale_size($$$) {
	my ($width, $height, $mode) = @_;
	if ($mode eq "thumb") {
		if ($width >= $height) {
			return ($thumb_width, $thumb_height);
		} else {
			return ($thumb_height, $thumb_width);
		}
	} else {
		if ($width >= $height) {
			return ($view_width, $view_height);
		} else {
			return ($view_height, $view_width);
		}
	}
}



=item make_filename
Make filename

0: directory
1: number
2: extension
r: filename
=cut
sub make_filename($$$) {
	my ($dir, $num, $ext) = @_;
	my ($n) = "00000000$num" =~ /(........)$/;
	$_ = "$dir$n$ext";
	s,//+,/,g;
	return $_;
}



=item make_cat_file
Make category filename

0: category
r: filename
=cut
sub make_cat_file($) {
	my ($cat) = @_;
	$cat =~ s/ /_/g;
	$cat =~ tr/A-Z/a-z/;
	$cat =~ s/[()]//g;
	return "cat_$cat.html";
}



=item make_view_pic_name
Make view picture filename

0: number
1: directory
2: filename
r: filepath
=cut
sub make_view_pic_name($$$) {
	my ($num, $dir, $file) = @_;
	return &make_filename("${dir}_r/view_", $num, "_$file");
}



=item make_view_html_name
Make view.html filename

0: picture number
r: filename
=cut
sub make_view_html_name($) {
	my ($num) = @_;
	return &make_filename("", $num, ".html");
}



=item make_thumb_name
Make thumbnail picture filename

0: number
1: directory
2: filename
r: filename
=cut
sub make_thumb_name($$$) {
	my ($num, $dir, $file) = @_;
	return &make_filename("${dir}_r/thumb_", $num, "_$file.png");
}



=item make_view_view
Create view picture

0: source
1: destination
=cut
sub make_view_pic($$) {
	my ($src, $dest) = @_;
	system("convert \"$src\" -scale ${view_true_size}x${view_true_size} " .
		"-despeckle -quality $view_quality \"$dest\"");
	print ".";
}



=item add_view
Add view to be created

0: number
1: directory
2: filename
3: rebuilding needed
=cut
sub add_view($$$$) {
	my ($num, $dir, $file, $rebuild) = @_;

	my ($id, $width, $height) = &get_file_info($dir, $file, $num);
	die "sanity check" if ($id != $num);

	my $real_name = "$dir/$file";
	my $view_name = &make_view_pic_name($num, $dir, $file);

	&make_view_pic($real_name, $view_name) if ($rebuild == 1);

	my $cats = &get_info($dir, $file, "category");
	my ($twidth, $theight) = &scale_size($width, $height, "thumb");
	&add_to_cats($file, $dir, $num, $cats, $twidth, $theight);
}



=item build_views
Create views
=cut
sub build_views() {
	foreach my $num (@file_id) {
		&build_view($num);
	}
}



=item parse_text
Parse text

0: text
r: text
=cut
sub parse_text($) {
	my ($text) = @_;
	my $fix;
	return undef if (! defined($text));
	foreach my $macro ($text =~ /(\[\[.*?\]\])/gs) {
		my ($in) = $macro =~ /\[\[\s*(.*?)\s*\]\]/s;
		my @param = split(/\] \[/, $in);
		my $repl;
		if ($#param == 1) {
			my $id = &cache_get_file_id($param[1]);
			if (! defined($id)) {
				print "bad marco: \[\[$macro\]\]\n";
				return $text;
			}
			my $view = &make_view_html_name($id);
			$repl = "<a href=\"$view\">$param[0]</a>";
		} else {
			print "bad macro: $macro\n";
			return $text;
		}
		$text =~ s/\Q$macro\E/$repl/s;
	}
	return $text;
}



=item build_view
Create view

0: id
=cut
sub build_view($) {
	my ($num) = @_;

	my $t;
	my $dir = $file_dir{$num};
	my $file = $file_name{$num};
	my ($id, $width, $height, $dummy) = &get_file_info($dir, $file, $num);
	my ($vwidth, $vheight) = &scale_size($width, $height, "view");

	my $real_name = "$dir/$file";
	my $view_name = &make_view_pic_name($num, $dir, $file);

	# title
	$tagval{'PICTURE-TITLE'} = &get_info($dir, $file, "title");

	if (! defined($tagval{'PICTURE-TITLE'})) {
		$tagval{'PICTURE-TITLE'} = $real_name;
	}

	print "$real_name\n";

	# description
	my $t_text = &get_info($dir, $file, "text");
	$t_text = &parse_text($t_text);
	$tagval{'PICTURE-DESCRIPTION'} = $t_text;

	# categories
	undef $tagval{'PICTURE-CATEGORIES'};
	my $cats = &get_info($dir, $file, "category");
	my @cats = &split_categories($cats);
	if ($#cats >= 0) {
		my $t = "";
		foreach (sort(@cats)) {
			my $cat = &make_cat_file($_);
			$t .= "<a href=\"$cat\">$_</a>\n";
		}
		$tagval{'PICTURE-CATEGORIES'} = $t;
	}

	# map
	undef $tagval{'PICTURE-MAP-URL'};
	my $t_map = &get_info($dir, $file, "map");
	$tagval{'PICTURE-MAP-LEVEL'} = 0;
	if (defined($t_map)) {
		$tagval{'PICTURE-MAP-URL'} = $t_map;
		$tagval{'PICTURE-MAP-LEVEL'} = 1;
	} else {
		my $t_map_ap = &get_info($dir, $file, "map-ap");
		if (defined($t_map_ap)) {
			$tagval{'PICTURE-MAP-URL'} = $t_map_ap;
			$tagval{'PICTURE-MAP-LEVEL'} = 2;
		}
	}

	# view
	$tagval{'PICTURE-VIEW'} =
		sprintf("<img src=\"%s\" width=\"%d\" height=\"%d\"\n" .
				"\talt=\"%s\" />",
			"$img_basepath$view_name",
			$vwidth, $vheight,
			$tagval{'PICTURE-TITLE'});
	$tagval{'PICTURE-VIEW-LINK-ORIG'} =
		"<a href=\"$img_basepath$real_name\">\n" .
		$tagval{'PICTURE-VIEW'} . "</a>";
	if ($num + 1 != $tagval{'TOTAL-PICTURES'}) {
		$t = &make_view_html_name($num + 1);
	} else {
		$t = $tagval{'LINK-INDEX-URL'};
	}
	$tagval{'PICTURE-VIEW-LINK-NEXT'} =
		"<a href=\"$t\">\n" . $tagval{'PICTURE-VIEW'} . "</a>";

	# prev/next
	undef $tagval{'VIEW-LINK-PREV-URL'};
	$tagval{'VIEW-LINK-PREV-LABEL'} = "&lt;&lt;&lt;";
	if ($num != 0) {
		$tagval{'VIEW-LINK-PREV-URL'} = &make_view_html_name($num - 1);
	}
	undef $tagval{'VIEW-LINK-NEXT-URL'};
	$tagval{'VIEW-LINK-NEXT-LABEL'} = "&gt;&gt;&gt;";
	if ($num + 1 != $tagval{'TOTAL-PICTURES'}) {
		$tagval{'VIEW-LINK-NEXT-URL'} = &make_view_html_name($num + 1);
	}

	# original
	$tagval{'PICTURE-LINK-ORIGINAL-URL'} = "$img_basepath$real_name";
	$tagval{'PICTURE-LINK-ORIGINAL-LABEL'} = "Original";

	# misc
	$tagval{'PICTURE-I'} = $num + 1;
	$tagval{'PICTURE-FULLPATH'} = "$img_basepath$real_name";
	$tagval{'PICTURE-PATH'} = "$img_basepath";
	$tagval{'PICTURE-FILE'} = "$real_name";


	# create page
	my $out_file = &make_view_html_name($num);
	my $page = $template_view;

	$page = &replace_tags($page) . "\n";

	open(FILE, "> $out_file") or die $@;
	print FILE $page;
	close(FILE);
}



=item make_thumb_pic
Make thumbnail picture

0: source
1: destination
=cut
sub make_thumb_pic($$) {
	my ($src, $dest) = @_;
	system("convert \"$src\" -dither " .
		"-scale ${thumb_true_size}x${thumb_true_size} \"$dest.gif\"");
	print ".";
	system("convert \"$dest.gif\" \"$dest\"");
	print ".";
	unlink("$dest.gif");
}



=item add_pic
Add picture to be processed

0: picture number
1: directory
2: filename
r: ID
=cut
sub add_pic($$$) {
	my ($num, $dir, $file) = @_;

	print "$dir/$file .";
	# use ID, it will be globally correct
	my ($id, $width, $height, $rebuild) = &get_file_info($dir, $file, $num);
	if ($rebuild == 1) {
		my $thumb = &make_thumb_name($id, $dir, $file);
		&make_thumb_pic("$dir/$file", $thumb);
	}

	&add_view($id, $dir, $file, $rebuild);
	print "\n";
	return $id;
}



=item add_dir
Add new directory to be scanned

0: first picture number to be assigned
1: directory
=cut
sub add_dir($$) {
	my ($num, $dir) = @_;
	print "dir: $dir\n";
	if ($cleanup == 1) {
		system("test -d \"${dir}_r/\" && rm -rf \"${dir}_r\"");
	}
	system("test -d \"${dir}_r/\" || mkdir \"${dir}_r\"");
	open(DPIPE, "find \"$dir/\" -iname '*.jp*g' | sort |") or die $@;
	while (<DPIPE>) {
		my ($file) = /.*\/(.*)/;
		$num = &add_pic($num, $dir, $file) + 1;
	}
	close(DPIPE);
}



=cut get_total_count
Get total count of pictures

@: directories
r: number of jpeg-pictures (with correct suffix)
=cut
sub get_total_count() {
	my $r;
	open(CPIPE, "find @_ -iname '*.jp*g' | wc -l |") or die $@;
	$r = <CPIPE>;
	close(CPIPE);
	chomp $r;
	return $r;
}



=item max
Get maximum of two

0: a
1: b
r: greater of a and b
=cut
sub max($$) {
	if ($_[0] > $_[1]) {
		return $_[0];
	} else {
		return $_[1];
	}
}



=item build_index
Build index page
=cut
sub build_index() {
	my $out_file = $main;
	my $page = $template_index;

	$page = &replace_tags($page);

	open(FILE, "> $out_file") or die $@;
	print FILE $page;
	close(FILE);
}



=item main
Main function

@: directories
=cut
sub main(@) {
	if ($#ARGV == -1) {
		print_usage();
		exit(0);
	}

	&parse_options();
	($thumb_width, $thumb_height) = $thumb_size =~ /(\d+)x(\d+)/;
	$thumb_true_size = &max($thumb_width, $thumb_height);
	($view_width, $view_height) = $view_size =~ /(\d+)x(\d+)/;
	$view_true_size = &max($view_width, $view_height);

	$tagval{'TOTAL-PICTURES'} = &get_total_count(@ARGV);
	$tagval{'LINK-INDEX-URL'} = $main; # XXX
	$tagval{'LINK-INDEX-LABEL'} = "Back to index";
	$tagval{'CREATED'} = localtime();
	my $id;

	if ($tagval{'TOTAL-PICTURES'} == 0) {
		print "Nothing to do.\n";
		return;
	}

	&cache_load("info_cache") if ($cleanup == 0);
	foreach (@ARGV) {
		$_ =~ s,/$,,;
		print "$_\n";
		$id = &get_new_id();
		&add_dir($id, $_);
	}
	&cache_save("info_cache");

	&get_tagval_all_thumbs();
	&get_tagval_all_categories();

	&build_views();
	&build_cats();
	&build_index();
}



&main(@ARGV);
